@@ -40,14 +40,13 @@ from zope.configuration.config import GroupingContextDecorator
 from zope.configuration.config import GroupingStackItem
 from zope.configuration.config import resolveConflicts
 from zope.configuration.exceptions import ConfigurationError
+from zope.configuration.exceptions import ConfigurationWrapperError
 from zope.configuration.fields import GlobalObject
 from zope.configuration.zopeconfigure import IZopeConfigure
 from zope.configuration.zopeconfigure import ZopeConfigure
 from zope.configuration._compat import reraise
 
 __all__ = [
-    'ZopeXMLConfigurationError',
-    'ZopeSAXParseException',
     'ParserInfo',
     'ConfigurationHandler',
     'processxmlfile',
@@ -70,40 +69,74 @@ ZCML_NAMESPACE = "http://namespaces.zope.org/zcml"
 ZCML_CONDITION = (ZCML_NAMESPACE, u"condition")
 
 
-class ZopeXMLConfigurationError(ConfigurationError):
-    """Zope XML Configuration error
-
-    These errors are wrappers for other errors. They include configuration
-    info and the wrapped error type and value.
+class ZopeXMLConfigurationError(ConfigurationWrapperError):
     """
-    def __init__(self, info, etype, evalue):
-        self.info, self.etype, self.evalue = info, etype, evalue
+    Zope XML Configuration error
 
-    def __str__(self):
-        # Only use the repr of the info. This is because we expect to
-        # get a parse info and we only want the location information.
-        return "%s\n    %s: %s" % (
-            repr(self.info), self.etype.__name__, self.evalue)
+    These errors are wrappers for other errors. They include
+    configuration info and the wrapped error type and value.
 
-class ZopeSAXParseException(ConfigurationError):
-    """Sax Parser errors, reformatted in an emacs friendly way
+    Example
+
+        >>> from zope.configuration.xmlconfig import ZopeXMLConfigurationError
+        >>> v = ZopeXMLConfigurationError("blah", AttributeError("xxx"))
+        >>> print(v)
+        'blah'
+            AttributeError: xxx
+    """
+
+    USE_INFO_REPR = True
+
+
+class ZopeSAXParseException(ConfigurationWrapperError):
+    """
+    Sax Parser errors as a ConfigurationError.
+
+    Example
+
+        >>> from zope.configuration.xmlconfig import ZopeSAXParseException
+        >>> v = ZopeSAXParseException("info", Exception("foo.xml:12:3:Not well formed"))
+        >>> print(v)
+        info
+            Exception: foo.xml:12:3:Not well formed
     """
-    def __init__(self, v):
-        self._v = v
 
-    def __str__(self):
-        v = self._v
-        s = tuple(str(v).split(':'))
-        if len(s) == 4:
-            return 'File "%s", line %s.%s, %s' % s
-        else:
-            return str(v)
 
 class ParserInfo(object):
-    """Information about a directive based on parser data
+    r"""
+    Information about a directive based on parser data
 
     This includes the directive location, as well as text data
     contained in the directive.
+
+    Example
+
+        >>> from zope.configuration.xmlconfig import ParserInfo
+        >>> info = ParserInfo('tests//sample.zcml', 1, 0)
+        >>> info
+        File "tests//sample.zcml", line 1.0
+
+        >>> print(info)
+        File "tests//sample.zcml", line 1.0
+
+        >>> info.characters("blah\n")
+        >>> info.characters("blah")
+        >>> info.text
+        'blah\nblah'
+
+        >>> info.end(7, 0)
+        >>> info
+        File "tests//sample.zcml", line 1.0-7.0
+
+        >>> print(info)
+        File "tests//sample.zcml", line 1.0-7.0
+          <configure xmlns='http://namespaces.zope.org/zope'>
+            <!-- zope.configure -->
+            <directives namespace="http://namespaces.zope.org/zope">
+              <directive name="hook" attributes="name implementation module"
+                 handler="zope.configuration.metaconfigure.hook" />
+            </directives>
+          </configure>
     """
     text = u''
 
@@ -173,7 +206,8 @@ class ParserInfo(object):
 
 
 class ConfigurationHandler(ContentHandler):
-    """Interface to the xml parser
+    """
+    Interface to the XML parser
 
     Translate parser events into calls into the configuration system.
     """
@@ -190,6 +224,16 @@ class ConfigurationHandler(ContentHandler):
     def characters(self, text):
         self.context.getInfo().characters(text)
 
+    def _handle_exception(self, ex, info):
+        if self.testing:
+            raise
+        if isinstance(ex, ConfigurationError):
+            ex.add_details(repr(info))
+            raise
+
+        exc = ZopeXMLConfigurationError(info, ex)
+        reraise(exc, None, sys.exc_info()[2])
+
     def startElementNS(self, name, qname, attrs):
         if self.ignore_depth:
             self.ignore_depth += 1
@@ -220,23 +264,82 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.begin(name, data, info)
-        except Exception:
-            if self.testing:
-                raise
-            reraise(ZopeXMLConfigurationError(info,
-                                              sys.exc_info()[0],
-                                              sys.exc_info()[1]),
-                    None, sys.exc_info()[2])
+        except Exception as ex:
+            self._handle_exception(ex, info)
 
         self.context.setInfo(info)
 
     def evaluateCondition(self, expression):
-        """Evaluate a ZCML condition.
+        """
+        Evaluate a ZCML condition.
 
         ``expression`` is a string of the form "verb arguments".
 
         Currently the supported verbs are ``have``, ``not-have``,
         ``installed`` and ``not-installed``.
+
+        The ``have`` and ``not-have`` verbs each take one argument:
+        the name of a feature:
+
+            >>> from zope.configuration.config import ConfigurationContext
+            >>> from zope.configuration.xmlconfig import ConfigurationHandler
+            >>> context = ConfigurationContext()
+            >>> context.provideFeature('apidoc')
+            >>> c = ConfigurationHandler(context, testing=True)
+            >>> c.evaluateCondition("have apidoc")
+            True
+            >>> c.evaluateCondition("not-have apidoc")
+            False
+            >>> c.evaluateCondition("have onlinehelp")
+            False
+            >>> c.evaluateCondition("not-have onlinehelp")
+            True
+
+        Ill-formed expressions raise an error:
+
+            >>> c.evaluateCondition("want apidoc")
+            Traceback (most recent call last):
+              ...
+            ValueError: Invalid ZCML condition: 'want apidoc'
+
+            >>> c.evaluateCondition("have x y")
+            Traceback (most recent call last):
+              ...
+            ValueError: Only one feature allowed: 'have x y'
+
+            >>> c.evaluateCondition("have")
+            Traceback (most recent call last):
+              ...
+            ValueError: Feature name missing: 'have'
+
+        The ``installed`` and ``not-installed`` verbs each take one
+        argument: the dotted name of a pacakge.
+
+        If the pacakge is found, in other words, can be imported, then
+        the condition will return true / false:
+
+            >>> context = ConfigurationContext()
+            >>> c = ConfigurationHandler(context, testing=True)
+            >>> c.evaluateCondition('installed zope.interface')
+            True
+            >>> c.evaluateCondition('not-installed zope.interface')
+            False
+            >>> c.evaluateCondition('installed zope.foo')
+            False
+            >>> c.evaluateCondition('not-installed zope.foo')
+            True
+
+        Ill-formed expressions raise an error:
+
+            >>> c.evaluateCondition("installed foo bar")
+            Traceback (most recent call last):
+              ...
+            ValueError: Only one package allowed: 'installed foo bar'
+
+            >>> c.evaluateCondition("installed")
+            Traceback (most recent call last):
+              ...
+            ValueError: Package name missing: 'installed'
         """
         arguments = expression.split(None)
         verb = arguments.pop(0)
@@ -286,13 +389,8 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.end()
-        except Exception:
-            if self.testing:
-                raise
-            reraise(ZopeXMLConfigurationError(info,
-                                              sys.exc_info()[0],
-                                              sys.exc_info()[1]),
-                    None, sys.exc_info()[2])
+        except Exception as ex:
+            self._handle_exception(ex, info)
 
 
 def processxmlfile(file, context, testing=False):
@@ -308,16 +406,58 @@ def processxmlfile(file, context, testing=False):
     try:
         parser.parse(src)
     except SAXParseException:
-        reraise(ZopeSAXParseException(sys.exc_info()[1]),
+        reraise(ZopeSAXParseException(file, sys.exc_info()[1]),
                 None, sys.exc_info()[2])
 
 
 def openInOrPlain(filename):
-    """Open a file, falling back to filename.in.
+    """
+    Open a file, falling back to filename.in.
 
     If the requested file does not exist and filename.in does, fall
-    back to filename.in.  If opening the original filename fails for
+    back to filename.in. If opening the original filename fails for
     any other reason, allow the failure to propogate.
+
+    For example, the tests/samplepackage dirextory has files:
+
+        - configure.zcml
+
+        - configure.zcml.in
+
+        - foo.zcml.in
+
+    If we open configure.zcml, we'll get that file:
+
+        >>> import os
+        >>> from zope.configuration.xmlconfig import __file__
+        >>> from zope.configuration.xmlconfig import openInOrPlain
+        >>> here = os.path.dirname(__file__)
+        >>> path = os.path.join(here, 'tests', 'samplepackage', 'configure.zcml')
+        >>> f = openInOrPlain(path)
+        >>> f.name[-14:]
+        'configure.zcml'
+        >>> f.close()
+
+    But if we open foo.zcml, we'll get foo.zcml.in, since there isn't
+    a foo.zcml:
+
+        >>> path = os.path.join(here, 'tests', 'samplepackage', 'foo.zcml')
+        >>> f = openInOrPlain(path)
+        >>> f.name[-11:]
+        'foo.zcml.in'
+        >>> f.close()
+
+    Make sure other IOErrors are re-raised. We need to do this in a
+    try-except block because different errors are raised on Windows
+    and on Linux.
+
+        >>> try:
+        ...     f = openInOrPlain('.')
+        ... except IOError:
+        ...     print("passed")
+        ... else:
+        ...     print("failed")
+        passed
     """
     try:
         return open(filename)
